@@ -107,6 +107,7 @@ typedef struct
   /* Internal Audio Widgets */
   GtkWidget      *audio_box;
   GtkScaleButton *audio_volume_button;
+  GtkLabel       *audio_duration_label;
   GtkLabel       *audio_position_label;
   GtkImage       *audio_playback_image;
 
@@ -167,6 +168,7 @@ enum
 {
   ERROR,
   SIZE_CHANGE,
+  TAGS_CHANGE,
   LAST_SIGNAL
 };
 
@@ -1212,6 +1214,17 @@ sushi_media_bin_class_init (SushiMediaBinClass *klass)
                     0, NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
+  /**
+   * SushiMediaBin::tags-change:
+   * @self: the #SushiMediaBin which received the signal.
+   */
+  sushi_media_bin_signals[TAGS_CHANGE] =
+      g_signal_new ("tags-change",
+                    G_TYPE_FROM_CLASS (object_class),
+                    G_SIGNAL_RUN_LAST,
+                    0, NULL, NULL, NULL,
+                    G_TYPE_NONE, 0);
+
   /* Action signals for key bindings */
   SMB_DEFINE_ACTION_SIGNAL (object_class, "toggle", sushi_media_bin_action_toggle, 1, G_TYPE_STRING);
   SMB_DEFINE_ACTION_SIGNAL (object_class, "seek", sushi_media_bin_action_seek, 1, G_TYPE_INT);
@@ -1237,6 +1250,7 @@ sushi_media_bin_class_init (SushiMediaBinClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, bottom_revealer);
 
   gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, audio_box);
+  gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, audio_duration_label);
   gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, audio_volume_button);
   gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, audio_position_label);
   gtk_widget_class_bind_template_child_private (widget_class, SushiMediaBin, audio_playback_image);
@@ -1457,6 +1471,7 @@ sushi_media_bin_update_duration (SushiMediaBin *self)
   priv->duration = duration;
 
   duration = GST_TIME_AS_SECONDS (duration);
+  gtk_label_set_label (priv->audio_duration_label, format_time (duration));
   gtk_label_set_label (priv->duration_label, format_time (duration));
   gtk_label_set_label (priv->progress_duration_label, format_time (duration));
   gtk_adjustment_set_upper (priv->playback_adjustment, duration);
@@ -1791,23 +1806,32 @@ sushi_media_bin_handle_msg_tag (SushiMediaBin *self, GstMessage *msg)
 
   if (g_type_is_a (G_OBJECT_TYPE (src), GST_TYPE_VIDEO_SINK))
     {
-      type = "video-tags-changed";
       old_tags = priv->video_tags;
-      priv->video_tags = gst_tag_list_merge (old_tags, tags, GST_TAG_MERGE_REPLACE);
+      if (!old_tags || (old_tags && !gst_tag_list_is_equal (old_tags, tags)))
+        {
+          type = "video-tags-changed";
+          priv->video_tags = gst_tag_list_merge (old_tags, tags, GST_TAG_MERGE_REPLACE);
+        }
     }
   else if (g_type_is_a (G_OBJECT_TYPE (src), GST_TYPE_AUDIO_BASE_SINK))
     {
-      type = "audio-tags-changed";
       old_tags = priv->audio_tags;
-      priv->audio_tags = gst_tag_list_merge (old_tags, tags, GST_TAG_MERGE_REPLACE);
+      if (!old_tags || (old_tags && !gst_tag_list_is_equal (old_tags, tags)))
+        {
+          type = "audio-tags-changed";
+          priv->audio_tags = gst_tag_list_merge (old_tags, tags, GST_TAG_MERGE_REPLACE);
+        }
     }
 
   /* Post message on the bus for the main thread to pick it up */
   if (type)
-    sushi_media_bin_post_message_application (self, type);
+    {
+      sushi_media_bin_post_message_application (self, type);
+      g_signal_emit (self, sushi_media_bin_signals[TAGS_CHANGE], 0);
+      g_clear_pointer (&old_tags, gst_tag_list_unref);
+    }
 
   gst_tag_list_unref (tags);
-  g_clear_pointer (&old_tags, gst_tag_list_unref);
 }
 
 static inline void
@@ -1815,8 +1839,8 @@ sushi_media_bin_handle_streams_selected (SushiMediaBin *self, GstMessage *msg)
 {
   SushiMediaBinPrivate *priv = SMB_PRIVATE (self);
   GstStreamCollection *collection = NULL;
+  GstStream *stream = NULL;
   GstStructure *caps_struct;
-  GstStream *stream;
   GstCaps *caps;
   gint i, n, w, h;
 
@@ -1831,6 +1855,9 @@ sushi_media_bin_handle_streams_selected (SushiMediaBin *self, GstMessage *msg)
         break;
     }
 
+  if (!stream)
+    return;
+
   caps = gst_stream_get_caps (stream);
   caps_struct = gst_caps_get_structure (caps, 0);
 
@@ -2100,6 +2127,26 @@ SMB_DEFINE_SETTER_FULL (gdouble, volume, VOLUME,
   gtk_adjustment_set_value (priv->volume_adjustment, volume),
 )
 
+/**
+ * sushi_media_bin_get_audio_tags:
+ * @self: a #SushiMediaBin
+ *
+ * Returns a #GstTagList with the audio tags for the played media
+ */
+SMB_DEFINE_GETTER_FULL (GstTagList *, audio_tags, NULL,
+  return (gst_tag_list_ref (SMB_PRIVATE (self)->audio_tags));
+)
+
+/**
+ * sushi_media_bin_get_video_tags:
+ * @self: a #SushiMediaBin
+ *
+ * Returns a #GstTagList with the video tags for the played media
+ */
+SMB_DEFINE_GETTER_FULL (GstTagList *, video_tags, NULL,
+  return (gst_tag_list_ref (SMB_PRIVATE (self)->video_tags));
+)
+
 /**
  * sushi_media_bin_play:
  * @self: a #SushiMediaBin
